@@ -1,6 +1,6 @@
-corSelect <- function(data, sp.cols = NULL, var.cols, coeff = TRUE, cor.thresh = ifelse(isTRUE(coeff), 0.8, 0.05), select = ifelse(is.null(sp.cols), "VIF", "p.value"), test = "Chisq", family = "auto", use = "pairwise.complete.obs", method = "pearson", verbosity = 1) {
+corSelect <- function(data, sp.cols = NULL, var.cols, coeff = TRUE, cor.thresh = ifelse(isTRUE(coeff), 0.8, 0.05), select = ifelse(is.null(sp.cols), "VIF", "cor"), test = "Chisq", family = "auto", use = "pairwise.complete.obs", method = "pearson", verbosity = 1) {
 
-  # version 3.6 (13 Mar 2024)
+  # version 3.8 (10 Dec 2025)
 
   if (length(sp.cols) > 1) stop ("Sorry, 'corSelect' is currently implemented for only one 'sp.col' at a time.")
 
@@ -76,7 +76,8 @@ corSelect <- function(data, sp.cols = NULL, var.cols, coeff = TRUE, cor.thresh =
 
     if (is.null(select))  return (high.cor.mat)
 
-    message("Using '", select, "' as the 'select' criterion.")
+    if (verbosity > 0)
+      message("Using '", select, "' as the 'select' criterion.")
 
     high.cor.vars <- unique(rownames(cor.mat[high.cor.inds, high.cor.inds]))
 
@@ -85,8 +86,12 @@ corSelect <- function(data, sp.cols = NULL, var.cols, coeff = TRUE, cor.thresh =
       cors <- cor(data.frame(data[ , sp.cols, drop = FALSE], data[ , match(high.cor.vars, colnames(data)), drop = FALSE]), use = "pairwise.complete.obs")[-1, 1]
       bivar.mat <- data.frame(bivar.mat, cor = cors[rownames(bivar.mat)])  # 'rownames' puts them in same order
       # if (isTRUE(all.equal(order(bivar.mat[ , c("p.value")]), order(bivar.mat[ , c("AIC")]), order(bivar.mat[ , c("BIC")]), order(abs(bivar.mat[ , c("cor")])), tolerance = 1.5e-8)))  # wrong use of 'all.equal'
-      if (isTRUE(all(sapply(list(order(bivar.mat[ , c("AIC")]), order(bivar.mat[ , c("BIC")]), order(abs(bivar.mat[ , c("cor")]))), FUN = identical, order(bivar.mat[ , c("p.value")])))))  # https://stackoverflow.com/a/30850654/3447652
-        message("Results identical whether 'select' is 'p.value', 'AIC', 'BIC' or cor.\n") else message("Results NOT identical whether 'select' is 'p.value', 'AIC', 'BIC' or 'cor'.\n")
+      if (verbosity > 0) {
+        if (isTRUE(all(sapply(list(order(bivar.mat[ , c("AIC")]), order(bivar.mat[ , c("BIC")]), order(abs(bivar.mat[ , c("cor")]))), FUN = identical, order(bivar.mat[ , c("p.value")])))))  # https://stackoverflow.com/a/30850654/3447652
+          message("Results identical whether 'select' is 'p.value', 'AIC', 'BIC' or cor.\n")
+        else
+          message("Results NOT identical whether 'select' is 'p.value', 'AIC', 'BIC' or 'cor'.\n")
+      }
     }  # end if select in bivar
 
     data.remaining <- data[ , var.cols]
