@@ -1,5 +1,9 @@
 favClass <- function(fav, breaks = c(0.2, 0.8), character = FALSE) {
   
+  # version 1.2 (2 May 2022)
+  
+  fav <- unlist(fav)
+  
   stopifnot(
     is.numeric(fav),
     is.numeric(breaks),
